@@ -11,6 +11,7 @@ import matplotlib.pyplot as plt
 from matplotlib.patches import Wedge
 from matplotlib.collections import PatchCollection
 import matplotlib.colors as colors
+from matplotlib.colors import Normalize, to_rgba, LinearSegmentedColormap
 import matplotlib.transforms as transforms
 import numpy as np
 import pandas as pd
@@ -18,6 +19,7 @@ import time
 from copy import deepcopy
 import os
 from scipy.ndimage import gaussian_filter
+from scipy.interpolate import interp1d
 
 import matplotlib as mpl
 mpl.rcParams.update(mpl.rcParamsDefault)
@@ -45,7 +47,22 @@ class Band:
             known by the user, as it was used to generate the KPOINTS file.
     """
 
-    def __init__(self, folder, projected=False, unfold=False, spin='up', kpath=None, n=None, M=None, high_symm_points=None, bandgap=False, printbg=True, shift_efermi=0):
+    def __init__(
+        self,
+        folder,
+        projected=False,
+        unfold=False,
+        spin='up',
+        kpath=None,
+        n=None,
+        M=None,
+        high_symm_points=None,
+        bandgap=False,
+        printbg=True,
+        shift_efermi=0,
+        interpolate=True,
+        new_n=200,
+    ):
         """
         Initialize parameters upon the generation of this class
 
@@ -66,6 +83,8 @@ class Band:
                 known by the user, as it was used to generate the KPOINTS file.
         """
 
+        self.interpolate = interpolate
+        self.new_n = new_n
         self.bandgap = bandgap
         self.printbg = printbg
         self.bg = None
@@ -147,8 +166,8 @@ class Band:
         self.orbital_labels = {
             0: 's',
             1: 'p_{y}',
-            2: 'p_{x}',
-            3: 'p_{z}',
+            2: 'p_{z}',
+            3: 'p_{x}',
             4: 'd_{xy}',
             5: 'd_{yz}',
             6: 'd_{z^{2}}',
@@ -192,7 +211,6 @@ class Band:
         self.bg = bg
 
 
-
     def _load_bands(self):
         """
         This function is used to load eigenvalues from the vasprun.xml
@@ -377,7 +395,6 @@ class Band:
 
         return projected_eigenvalues
 
-    
 
     def _sum_spd(self, spd):
         """
@@ -608,7 +625,7 @@ class Band:
         ax.set_xticks([wave_vectors[k] for k in kpoints_index])
         ax.set_xticklabels(kpts_labels)
 
-    def _get_kticks_hse(self, ax, kpath, vlinecolor):
+    def _get_kticks_hse(self, wave_vectors, ax, kpath, vlinecolor):
         structure = self.poscar.structure
         kpath_obj = HighSymmKpath(structure)
         kpath_labels = np.array(list(kpath_obj._kpath['kpoints'].keys()))
@@ -625,12 +642,12 @@ class Band:
 
         kpoints_index = index
         kpath = kpath_labels[label_index]
-        kpoints_index = ax.lines[0].get_xdata()[kpoints_index]
+        #  kpoints_index = ax.lines[0].get_xdata()[kpoints_index]
 
         kpath = [f'${k}$' if k != 'G' else '$\\Gamma$' for k in kpath]
 
         for k in kpoints_index:
-            ax.axvline(x=k, color=vlinecolor, alpha=0.7, linewidth=0.5)
+            ax.axvline(x=wave_vectors[k], color=vlinecolor, alpha=0.7, linewidth=0.5)
 
         plt.xticks(kpoints_index, kpath)
 
@@ -651,6 +668,50 @@ class Band:
         ax.set_xticklabels(kpath)
         #  plt.xticks(np.array(kpoints)[kpoints_index], kpath)
 
+    def _get_slices(self, unfold=False, hse=False):
+        if not unfold and not hse:
+            high_sym_points = self.kpoints_file.kpts
+            all_kpoints = self.kpoints
+
+            index = [0]
+            for i in range(len(high_sym_points) - 2):
+                if high_sym_points[i + 2] != high_sym_points[i + 1]:
+                    index.append(i)
+            index.append(len(high_sym_points) - 1)
+            kpts_loc = np.isin(np.round(all_kpoints, 3), np.round(high_sym_points, 3)).all(1)
+            index = np.where(kpts_loc == True)[0][index]
+            index[0] = -1
+            slices = [np.s_[index[i]+1:index[i+1]+1] for i in range(0,len(index)-1)]
+
+        if hse and not unfold:
+            structure = self.poscar.structure
+            kpath_obj = HighSymmKpath(structure)
+            kpath_coords = np.array(list(kpath_obj._kpath['kpoints'].values()))
+            index = np.where((self.kpoints[:, None] == kpath_coords).all(-1).any(-1) == True)[0]
+            index = [index[0]] + [index[i] for i in range(1,len(index)-1) if i % 2] + [index[-1]]
+            slices = [np.s_[index[i]:index[i+1]+1] for i in range(len(index)-1)]
+
+        if unfold and not hse:
+            index = [0] + [(self.n * i) for i in range(1, len(self.kpath))]
+            slices = [np.s_[index[i]:index[i+1]+1] for i in range(len(index)-1)]
+
+        return slices
+
+    def _get_interpolated_data(self, wave_vectors, data, crop_zero=False, kind='cubic'):
+        slices = self._get_slices(unfold=self.unfold, hse=self.hse)
+        data = [data[:,i] for i in slices]
+        wave_vectors = [wave_vectors[i] for i in slices]
+        fs = [interp1d(i, j, kind=kind, axis=1) for (i,j) in zip(wave_vectors, data)]
+        new_wave_vectors = [np.linspace(wv.min(), wv.max(), self.new_n) for wv in wave_vectors]
+        data = np.hstack([f(wv) for (f, wv) in zip(fs, new_wave_vectors)])
+        wave_vectors = np.hstack(new_wave_vectors)
+
+        if crop_zero:
+            data[np.where(data < 0)] = 0
+
+        return wave_vectors, data
+
+
     def _filter_bands(self, erange):
         eigenvalues = self.eigenvalues
         where = (eigenvalues >= np.min(erange) - 1) & (eigenvalues <= np.max(erange) + 1)
@@ -744,6 +805,19 @@ class Band:
             norm=norm,
         )
 
+    def _alpha_cmap(color):
+        cmap = LinearSegmentedColormap.from_list(
+            'custom_cmap',
+            [
+                (1,1,1,0),
+                to_rgba(color),
+                to_rgba(color),
+            ],
+            N=10000
+        )
+        return cmap
+        
+
 
     def plot_plain(
         self,
@@ -774,12 +848,26 @@ class Band:
         bands_in_plot = self._filter_bands(erange=erange)
         eigenvalues = self.eigenvalues[bands_in_plot]
         wave_vectors = self._get_k_distance()
+        wave_vectors_for_kpoints = wave_vectors
+
+        if self.interpolate:
+            wave_vectors, eigenvalues = self._get_interpolated_data(wave_vectors_for_kpoints, eigenvalues)
+
         eigenvalues_ravel = np.ravel(np.c_[eigenvalues, np.empty(eigenvalues.shape[0]) * np.nan])
         wave_vectors_tile = np.tile(np.append(wave_vectors, np.nan), eigenvalues.shape[0])
 
         if self.unfold:
             spectral_weights = self.spectral_weights[bands_in_plot]
             spectral_weights = spectral_weights / np.max(spectral_weights)
+
+            if self.interpolate:
+                _, spectral_weights = self._get_interpolated_data(
+                    wave_vectors_for_kpoints,
+                    spectral_weights,
+                    crop_zero=True,
+                    kind='linear',
+                )
+            
             spectral_weights_ravel = np.ravel(np.c_[spectral_weights, np.empty(spectral_weights.shape[0]) * np.nan])
 
             if heatmap:
@@ -805,21 +893,35 @@ class Band:
                     zorder=0,
                 )
         else:
-            ax.plot(
-                wave_vectors_tile,
-                eigenvalues_ravel,
-                color=color,
-                linewidth=linewidth,
-                linestyle=linestyle,
-                zorder=0,
-            )
+            if heatmap:
+                self._heatmap(
+                    ax=ax,
+                    wave_vectors=wave_vectors,
+                    eigenvalues=eigenvalues,
+                    weights=np.ones(eigenvalues.shape),
+                    sigma=sigma,
+                    cmap=cmap,
+                    bins=bins,
+                    projection=projection,
+                    powernorm=powernorm,
+                    gamma=gamma,
+                )
+            else:
+                ax.plot(
+                    wave_vectors_tile,
+                    eigenvalues_ravel,
+                    color=color,
+                    linewidth=linewidth,
+                    linestyle=linestyle,
+                    zorder=0,
+                )
 
         if self.hse:
-            self._get_kticks_hse(ax=ax, kpath=self.kpath, vlinecolor=vlinecolor)
+            self._get_kticks_hse(ax=ax, wave_vectors=wave_vectors_for_kpoints, kpath=self.kpath, vlinecolor=vlinecolor)
         elif self.unfold:
-            self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors, vlinecolor=vlinecolor)
+            self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors_for_kpoints, vlinecolor=vlinecolor)
         else:
-            self._get_kticks(ax=ax, wave_vectors=wave_vectors, vlinecolor=vlinecolor)
+            self._get_kticks(ax=ax, wave_vectors=wave_vectors_for_kpoints, vlinecolor=vlinecolor)
 
         ax.set_xlim(0, np.max(wave_vectors))
 
@@ -841,6 +943,7 @@ class Band:
         vlinecolor='black',
         powernorm=False,
         gamma=0.5,
+        plain_scale_factor=10,
     ):
         """
         This is a general method for plotting projected data
@@ -861,14 +964,37 @@ class Band:
 
         bands_in_plot = self._filter_bands(erange=erange)
         projected_data = projected_data[bands_in_plot]
-        projected_data = projected_data / np.max(projected_data)
+        unique_colors = np.unique(colors)
+        shapes = (projected_data.shape[0], projected_data.shape[1], projected_data.shape[-1])
+        projected_data = projected_data.reshape(shapes)
+
+        if len(unique_colors) == len(colors):
+            pass
+        else:
+            unique_inds = [np.isin(colors, c) for c in unique_colors]
+            projected_data = np.squeeze(projected_data)
+            projected_data = np.c_[
+                [np.sum(projected_data[...,i], axis=2) for i in unique_inds]
+            ].transpose((1,2,0))
+            colors = unique_colors
+
+        #  projected_data = projected_data / np.max(projected_data)
         wave_vectors = self._get_k_distance()
+        wave_vectors_old = wave_vectors
         eigenvalues = self.eigenvalues[bands_in_plot]
 
-        if heatmap:
-            if self.unfold:
-                K_indices = np.array(self.K_indices[0], dtype=int)
-                projected_data = projected_data[:, K_indices, :]
+        if self.unfold:
+            K_indices = np.array(self.K_indices[0], dtype=int)
+            projected_data = projected_data[:, K_indices, :]
+
+        if self.interpolate:
+            wave_vectors, eigenvalues = self._get_interpolated_data(wave_vectors_old, eigenvalues)
+            _, projected_data = self._get_interpolated_data(
+                wave_vectors_old,
+                projected_data,
+                crop_zero=True,
+                kind='linear',
+            )
 
         self.plot_plain(
             ax=ax,
@@ -881,14 +1007,22 @@ class Band:
             bins=bins,
             vlinecolor=vlinecolor,
             projection=projected_data,
+            scale_factor=plain_scale_factor,
         )
 
         if not heatmap:
             if self.unfold:
                 spectral_weights = self.spectral_weights[bands_in_plot]
                 spectral_weights = spectral_weights / np.max(spectral_weights)
-                K_indices = np.array(self.K_indices[0], dtype=int)
-                projected_data = projected_data[:, K_indices, :]
+
+                if self.interpolate:
+                    _, spectral_weights = self._get_interpolated_data(
+                        wave_vectors_old,
+                        spectral_weights,
+                        crop_zero=True,
+                        kind='linear',
+                    )
+
                 spectral_weights_ravel = np.repeat(np.ravel(spectral_weights), projected_data.shape[-1])
 
             projected_data_ravel = np.ravel(projected_data)
@@ -1520,59 +1654,60 @@ class Band:
 
 
 if __name__ == "__main__":
-    M = [
-        [0,1,-1],
-        [1,-1,0],
-        [-14,-14,-14]
-    ]
-
-    high_symm_points = [
-        [2/3, 1/3, 1/3],
-        [0.0, 0.0, 0],
-        [2/3, 1/3, 1/3],
-    ]
-
-    #  high_symm_points = [
-        #  [0.1, 0.1, 0],
-        #  [0.0, 0.0, 0],
-        #  [0.1, 0.1, 0],
+    pass
+    #  M = [
+        #  [0,1,-1],
+        #  [1,-1,0],
+        #  [-14,-14,-14]
     #  ]
-
-    band = Band(
-        folder="../../vaspvis_data/bandMGM",
-        projected=False,
-        unfold=True,
-        kpath='AGA',
-        high_symm_points=high_symm_points,
-        n=40,
-        M=M,
-    )
-    fig, ax = plt.subplots(figsize=(3,4), dpi=300)
-    start = time.time()
-    #  band.plot_plain(ax=ax, color=[(0.9,0.9,0.9)])
-    #  band.plot_spd(ax=ax, orbitals='sd', display_order='all', scale_factor=35, erange=[-5,0])
-    #  band.plot_orbitals(ax=ax, scale_factor=35, orbitals=range(8), display_order=None)
-    band.plot_plain( 
-        ax=ax,
-        scale_factor=20,
-        erange=[-4,0.5],
-        heatmap=True,
-        cmap='hot',
-        bins=800,
-        sigma=3,
-        powernorm=False,
-        gamma=0.5,
-        vlinecolor='white'
-    )
-    #  ax.set_aspect(3, adjustable='datalim')
-    end = time.time()
-    print(end-start)
-    ax.set_ylabel('$E - E_{F}$ $(eV)$', fontsize=6)
-    ax.tick_params(labelsize=6, length=2.5)
-    ax.tick_params(axis='x', length=0)
-    ax.set_ylim(-4, 0.5)
-    plt.tight_layout(pad=0.2)
-    plt.savefig('heatmap_powernorm.png')
+#
+    #  high_symm_points = [
+        #  [2/3, 1/3, 1/3],
+        #  [0.0, 0.0, 0],
+        #  [2/3, 1/3, 1/3],
+    #  ]
+#
+    #  #  high_symm_points = [
+        #  #  [0.1, 0.1, 0],
+        #  #  [0.0, 0.0, 0],
+        #  #  [0.1, 0.1, 0],
+    #  #  ]
+#
+    #  band = Band(
+        #  folder="../../vaspvis_data/bandMGM",
+        #  projected=False,
+        #  unfold=True,
+        #  kpath='AGA',
+        #  high_symm_points=high_symm_points,
+        #  n=40,
+        #  M=M,
+    #  )
+    #  fig, ax = plt.subplots(figsize=(3,4), dpi=300)
+    #  start = time.time()
+    #  #  band.plot_plain(ax=ax, color=[(0.9,0.9,0.9)])
+    #  #  band.plot_spd(ax=ax, orbitals='sd', display_order='all', scale_factor=35, erange=[-5,0])
+    #  #  band.plot_orbitals(ax=ax, scale_factor=35, orbitals=range(8), display_order=None)
+    #  band.plot_plain(
+        #  ax=ax,
+        #  scale_factor=20,
+        #  erange=[-4,0.5],
+        #  heatmap=True,
+        #  cmap='hot',
+        #  bins=800,
+        #  sigma=3,
+        #  powernorm=False,
+        #  gamma=0.5,
+        #  vlinecolor='white'
+    #  )
+    #  #  ax.set_aspect(3, adjustable='datalim')
+    #  end = time.time()
+    #  print(end-start)
+    #  ax.set_ylabel('$E - E_{F}$ $(eV)$', fontsize=6)
+    #  ax.tick_params(labelsize=6, length=2.5)
+    #  ax.tick_params(axis='x', length=0)
+    #  ax.set_ylim(-4, 0.5)
+    #  plt.tight_layout(pad=0.2)
+    #  plt.savefig('heatmap_powernorm.png')
         
         
 
